;**************************************************************************
;**
;** FILE  intro.asm
;** Copyright (c) 1994, 2001 Daniel Kahlin
;** Written by Daniel Kahlin <daniel@kahlin.net>
;**
;** DESCRIPTION
;**   RasterRoutine 1.0 - Daniel "T.L.R" Kahlin
;**   Alpha Range 89 Intro
;**
;******
	PROCESSOR 6502

VB_BGCOL		EQU	0
LOGO_KANTCOL		EQU	1
LOGO_BGCOL		EQU	4
LOGO_RAMCOL		EQU	0
REST_BGCOL		EQU	6
SCROLL_BGCOL		EQU	6
SCROLL_TEXTCOL		EQU	1
TEXTAREA_TEXTCOL	EQU	1

BLANKCHAR		EQU	6

; show raster time
RASTERTIME		EQU	0
;*** Flags ***

MUSIC		EQU	0

;Debug Stab
DEBUG		EQU	0

;Set to disable RESTORE
RESTOREDISABLE	EQU	1



;the length of a screen in cycles
SCREENTIME	EQU	22150

;the length of a row in cycles
ROWTIME		EQU	71

;the time to setting the next interrupt
COMPTIME	EQU	30+71



;Macro to enable next interrupt within this one
	MAC	FORK
	pla
	pla
	jsr	Update
	cli
	ENDM



MultBuff	EQU	$f7	;8	bytes


	seg	code
	org	$2000


;**************************************************************************
;**
;** SysAddress... When run we will enter here!
;**
;******
SysAddress:
	sei
	jsr	InitScrollBar	
	jsr	VideoInit
	jsr	InterruptInit
	IF	MUSIC
	jsr	pl_Init
	ENDIF
	cli
sa_lp1:
	lda	exitflag
	beq	sa_lp1

	IF	RESTOREDISABLE
	lda	#%00000010
	sta	$911d		; Clear pending restore keys
	ENDIF

	sei
	jsr	$fd52		; set vectors
	jsr	$fdf9		; set timer ints
	jsr	$e518		; init screen

	rts

exitflag:
	dc.b	0
	
;**************************************************************************
;**
;** Initialize Video!
;**
;******
VideoInit:
	jsr	Stab

	lda	#$0c+4	;horisontal (ingen interlace)
	sta	$9000
	lda	#38	;Vertikal
	sta	$9001	
	lda	#$00+18	;kolumner
	sta	$9002
	lda	#23*2+0	;rader
	sta	$9003
	lda	#$ce
	sta	$9005
	lda	#$d0
	sta	$900e
	lda	#$08+LOGO_BGCOL
	sta	$900f

	ldx	#0
vi_lp1:
	lda	#LOGO_RAMCOL+8
	sta	$9400,x
	sta	$9500,x
	lda	#BLANKCHAR
	sta	$1000,x
	sta	$1100,x
	inx
	bne	vi_lp1

	lda	#0
	sta	Offset1

	jsr	Showtext
	jsr	Showscroll

	rts




;**************************************************************************
;**
;** Initialize Interrupts!
;**
;******
InterruptInit:
	IF	RESTOREDISABLE
	lda	#%00000010
	sta	$911e	;No RestoreKey
	sta	$911d
	ENDIF

	lda	#$7f
	sta	$912e
	sta	$912d
	lda	#%11100000
	sta	$912e
	lda	#%01000000
	sta	$912b
	lda	#<IRQServer
	sta	$0314
	lda	#>IRQServer
	sta	$0315

	jsr	Stab	;Find Fixpoint!!! (VertBlank)
	lda	#<SCREENTIME
	sta	$9124
	lda	#>SCREENTIME
	sta	$9125	;load T1
	rts


;**************************************************************************
;**
;** Stabilize routine
;**
;******
Stab:

st_lp4
	lda	$9004
	bne	st_lp4
st_lp5:
	lda	$9004
	beq	st_lp5


	ldy	#64
st_lp1:
	lda	$9003
	bmi	st_skp1
st_skp1:
	IF	DEBUG
	ldx	#$77+8
	stx	$900f
	ldx	#$66+8
	stx	$900f	;18
	nop
	nop
	nop
	ldx	#9	;1+5*9
	ELSE
	bit	$ea
	ldx	#12
	ENDIF
st_lp2:
	dex
	bne	st_lp2

	lda	$9003
	bpl	st_skp2
st_skp2:
	IF	DEBUG
	ldx	#$77+8
	stx	$900f
	ldx	#$66+8
	stx	$900f	;18
	nop
	nop
	nop
	ldx	#8	;1+5*9
	ELSE
	bit	$ea
	ldx	#11
	ENDIF
st_lp3:
	dex
	bne	st_lp3

	dey
	bne	st_lp1

	nop
st_lp6:
	lda	$9004
	beq	st_ex1
	IF	DEBUG
	ldx	#$11+8
	stx	$900f
	ldx	#$66+8
	stx	$900f	;18
	nop
	nop
	nop
	ldx	#8	;1+5*9
	ELSE
	bit	$ea
	ldx	#11
	ENDIF
st_lp7:
	dex
	bne	st_lp7
	bit	$ea
	jmp	st_lp6

st_ex1:
	rts	
StabEnd:
	ECHO	"Stab... ",Stab,StabEnd




;**************************************************************************
;**
;** Wait until line ACC is passed
;**
;******
WaitLine:
s_lp2:
	cmp	$9004
	bne	s_lp2
s_lp3:
	cmp	$9004
	beq	s_lp3
	rts

;**************************************************************************
;**
;** Multiply Acc*71	(A * %01000111)
;**
;******
Mult71:
	sta	MultBuff+0
	sty	MultBuff+1
	sty	MultBuff+3
	asl
	sta	MultBuff+2
	rol	MultBuff+3
	ldy	MultBuff+3
	sty	MultBuff+5
	asl
	sta	MultBuff+4
	rol	MultBuff+5
	ldy	MultBuff+5
	sty	MultBuff+7
	asl
	rol	MultBuff+7
	asl
	rol	MultBuff+7
	asl
	rol	MultBuff+7
	asl
	rol	MultBuff+7
	sta	MultBuff+6

	lda	MultBuff+6
	clc
	adc	MultBuff+4
	sta	MultBuff+4
	lda	MultBuff+7
	adc	MultBuff+5
	sta	MultBuff+5

	lda	MultBuff+4
	clc
	adc	MultBuff+2
	sta	MultBuff+2
	lda	MultBuff+5
	adc	MultBuff+3
	sta	MultBuff+3

	lda	MultBuff+2
	clc
	adc	MultBuff+0
	sta	MultBuff+0
	lda	MultBuff+3
	adc	MultBuff+1
	sta	MultBuff+1
	tay
	lda	MultBuff+0
	rts



;**************************************************************************
;**
;** IRQ Interrupt server
;**
;******
IRQServer:
	lda	$912d
	asl
	asl
	bcs	irq_skp1
	asl
	bcc	irq_ex1
	jsr	T2Timeout
	jmp	irq_ex1
irq_skp1:
	jsr	T1Timeout
irq_ex1:
	pla
	tay
	pla
	tax
	pla
	rti



;**************************************************************************
;**
;** Routines + stuff
;**
;******
T1Timeout:
	lda	#[75+[<t1_TComp]]&255
	sec
	sbc	$9124	;68-73
	sta	t1_SelfModJMP+1
t1_SelfModJMP:
	jmp	t1_TComp
t1_TComp:
	dc.b	$a9,$a9,$a9,$a9,$a9,$a9
	dc.b	$a9,$a9,$a9,$a9,$24,$ea
t1_TCompEnd:
	ECHO	"T1COMP... ",t1_TComp,t1_TCompEnd

	lda	InterruptTab
	sta	$9128
	lda	InterruptTab+1
	sta	$9129	;load T2
	lda	#$00
	sta	Line
	jsr	Update
	lda	#[VB_BGCOL<<4]+VB_BGCOL+8
	sta	$900f
	rts


T2Timeout:
	lda	#[$c0+[<t2_TComp]]&255
	sec
	sbc	$9128	;68-73
	sta	t2_SelfModJMP+1
t2_SelfModJMP:
	jmp	t2_TComp
t2_TComp:
	dc.b	$a9,$a9,$a9,$a9,$a9,$a9
	dc.b	$a9,$a9,$a9,$a9,$24,$ea
t2_TCompEnd:
	ECHO	"T2COMP... ",t2_TComp,t2_TCompEnd

	lda	TimeLow
	sta	$9128
	lda	TimeHigh
	sta	$9129

SelfModJSR:
	jsr	SelfModJSR

Update:
	lda	Line
	inc	Line
	asl
	asl
	tax
	lda	InterruptTab+4,x
	sta	TimeLow
	lda	InterruptTab+5,x
	sta	TimeHigh
	lda	InterruptTab+2,x
	sta	SelfModJSR+1
	lda	InterruptTab+3,x
	sta	SelfModJSR+2
	rts


TimeLow:
	dc.b	0
TimeHigh:
	dc.b	0
Line:
	dc.b	0
InterruptTab:
	dc.w	ROWTIME*69-COMPTIME+12,Rast0
	dc.w	ROWTIME*5-COMPTIME+44,Rast1
	dc.w	ROWTIME*77-COMPTIME-44,Rast2
	dc.w	ROWTIME*10-COMPTIME,Rast3
	dc.w	ROWTIME*33-COMPTIME,Rast4
	dc.w	ROWTIME*19-COMPTIME,Rast5
	dc.w	ROWTIME*21-COMPTIME,Rast6
	dc.w	ROWTIME*11-COMPTIME,Rast7
	dc.w	-1,-1




;**************************************************************************
;**
;** Routines + stuff
;**
;******



;**************************************************************************
;**
;** LOGO
;**
;******
buflen	EQU	72

	org	$2300
;*** Gr Logostart ***
Rast0:
	lda	#[LOGO_KANTCOL<<4]+LOGO_KANTCOL+8
	sta	$900f
	ldx	#12
r1_lp1:
	dex
	bne	r1_lp1
	nop
	nop
	lda	#$08+LOGO_BGCOL
	sta	$900f

	lda	#4	;horisontal (ingen interlace)
	sta	$9000
	lda	#30	;kolumner
	sta	$9002
	lda	#$ce
	sta	$9005
	rts
;*** Visa colorbars i logo ***
Rast1:
	ldx	#0
	lda	$900e
	and	#$0f
	sta	temp
cb_lp1:
	lda	Buffer1+1,x
	sta	Buffer1,x
	ora	temp
	ldy	Buffer2,x
	sta	$900e
	sty	$900f
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	bit	$ea
	inx
	cpx	#buflen-1
	bne	cb_lp1

	lda	#$08+LOGO_BGCOL
	sta	$900f
	lda	temp
	sta	$900e
	rts

;*** Gr logoslut och scrolla frger ***
Rast2:
	lda	#[LOGO_KANTCOL<<4]+LOGO_KANTCOL+8
	sta	$900f
	ldx	#12
r2_lp1:
	dex
	bne	r2_lp1
	nop
	nop
	lda	#[VB_BGCOL<<4]+VB_BGCOL+8
	sta	$900f

	ldx	#15
r2_lp2:
	dex
	bne	r2_lp2
	lda	#0	;kolumner
	sta	$9002
	rts


;**************************************************************************
;**
;** Text area
;**
;******

;*** Starta textarea ***
Rast3:
	lda	#12	;horisontal (ingen interlace)
	sta	$9000
	lda	#22	;kolumner
	sta	$9002

	rts

;*** Avsluta textarea ***
Rast4:

	lda	#0	;kolumner
	sta	$9002
	rts



;**************************************************************************
;**
;** SCROLL
;**
;******

;*** Starta scroll ***
Rast5:
	lda	#[LOGO_KANTCOL<<4]+LOGO_KANTCOL+8
	sta	$900f
	ldx	#12
r5_lp1:
	dex
	bne	r5_lp1
	nop
	nop
	lda	#[SCROLL_BGCOL<<4]+SCROLL_BGCOL+8
	sta	$900f

	
	lda	#4	;horisontal (ingen interlace)
	sta	$9000
	lda	#30	;kolumner
	sta	$9002
	rts

;*** Avsluta scroll ***
Rast6:
	lda	#0	;kolumner
	sta	$9002
	rts
	
;*** Avsluta scroll ***
Rast7:
	lda	#[LOGO_KANTCOL<<4]+LOGO_KANTCOL+8
	sta	$900f
	ldx	#12
r7_lp1:
	dex
	bne	r7_lp1
	nop
	nop
	lda	#[VB_BGCOL<<4]+VB_BGCOL+8
	sta	$900f

	IF	RASTERTIME
	lda	$900f
	pha
	lda	#$ff
	sta	$900f
	ENDIF
	jsr	LogoScroll1
	jsr	ScrollBar
	jsr	doscroll

	IF	MUSIC
	jsr	pl_Play
	ENDIF
	IF	RASTERTIME
	pla
	sta	$900f
	ENDIF

; check for space
	lda	#$ef
	sta	$9120
	lda	$9121
	cmp	#$fe
	bne	r7_ex1
	lda	#1
	sta	exitflag
r7_ex1:	
	rts





;**************************************************************************
;**
;** Scroll ColorBars
;**
;******
InitScrollBar:	
	ldx	#buflen-1
isb_lp1:
	txa
	pha

	ldx	#0
isb_lp2:
	lda	Buffer1+1,x
	sta	Buffer1,x
	inx
	cpx	#buflen-1
	bne	isb_lp2
	jsr	ScrollBar

	pla
	tax
	dex
	bpl	isb_lp1
	rts
	
ScrollBar:	
	lda	frame
	eor	#$ff
	sta	frame
	beq	cb_skp6
	ldx	#buflen-2
cb_lp4:
	lda	Buffer2,x
	sta	Buffer2+1,x
	dex
	bpl	cb_lp4
cb_skp6:

	lda	Cnt1
	lsr
	lsr
	lsr
	lsr
	lsr
	tax
cb_lp2:
	lda	Tab1,x
	bpl	cb_skp1
	ldx	#0
	stx	Cnt1
	beq	cb_lp2
cb_skp1:
	asl
	asl
	asl
	asl
	sta	Buffer1+buflen-1
	inc	Cnt1


	lda	Cnt2
	lsr
	tax
cb_lp3:
	lda	Tab2,x
	bpl	cb_skp2
	ldx	#0
	stx	Cnt2
	beq	cb_lp3
cb_skp2:
	asl
	asl
	asl
	asl
	ora	#LOGO_BGCOL
	sta	Buffer2
	
	inc	Cnt2
	rts
Cnt1:
	dc.b	0
Cnt2:
	dc.b	0
temp:
	dc.b	0
frame:
	dc.b	0

;**************************************************************************
;**
;** LogoScroll1  (Alpha)
;**
;******
LogoScroll1:
	lda	Offset1
	cmp	#18+6
	beq	ls1_ex1

	ldx	#0
ls1_lp1:
	lda	$1001,x
	sta	$1000,x
	inx
	cpx	#[30*3]-1
	bne	ls1_lp1

	lda	Offset1
	inc	Offset1

	cmp	#18
	bcs	ls1_skp1
	sta	$1000+[30*0]+29
	clc
	adc	#18
	sta	$1000+[30*1]+29
	clc
	adc	#18
	sta	$1000+[30*2]+29
	rts
ls1_skp1:
	lda	#6
	sta	$1000+[30*0]+29
	sta	$1000+[30*1]+29
	sta	$1000+[30*2]+29
	rts
ls1_ex1:
	jsr	LogoScroll2
	rts
Offset1:
	dc.b	0


;**************************************************************************
;**
;** LogoScroll2  (Range)
;**
;******
LogoScroll2:
	lda	Offset2
	cmp	#18+6
	beq	ls2_ex1

	ldx	#[30*3]-2
ls2_lp1:
	lda	$1000+[30*3],x
	sta	$1001+[30*3],x
	dex
	bpl	ls2_lp1

	lda	Offset2
	inc	Offset2
	cmp	#18
	bcs	ls2_skp1

	lda	#18+[18*3]
	sec
	sbc	Offset2
	sta	$1000+[30*3]
	clc
	adc	#18
	sta	$1000+[30*4]
	clc
	adc	#18
	sta	$1000+[30*5]
	rts
ls2_skp1:
	lda	#6
	sta	$1000+[30*3]
	sta	$1000+[30*4]
	sta	$1000+[30*5]
	rts
ls2_ex1:
	jsr	LogoScroll3
	rts
Offset2:
	dc.b	0


;**************************************************************************
;**
;** LogoScroll3  (89)
;**
;******
LogoScroll3:
	lda	Offset3
	cmp	#18+6
	beq	ls3_ex1

	ldx	#0
ls3_lp1:
	lda	$1001+[30*6],x
	sta	$1000+[30*6],x
	inx
	cpx	#[30*3]-1
	bne	ls3_lp1

	lda	Offset3
	inc	Offset3

	cmp	#18
	bcs	ls3_skp1

	clc
	adc	#[18*6]
	sta	$1000+[30*6]+29
	clc
	adc	#18
	sta	$1000+[30*7]+29
	clc
	adc	#18
	sta	$1000+[30*8]+29
	rts
ls3_skp1:
	lda	#6
	sta	$1000+[30*6]+29
	sta	$1000+[30*7]+29
	sta	$1000+[30*8]+29
	rts
ls3_ex1:
	lda	#1
	sta	scrollflag
	rts
Offset3:
	dc.b	0


;**************************************************************************
;**
;** Display text area
;**
;******
Showtext:
	ldx	#[22*3]-1
sht_lp1:
	lda	string,x
	and	#$3f
	ora	#$c0
	sta	$1000+[30*10]+[22*1],x
	lda	#TEXTAREA_TEXTCOL
	sta	$9400+[30*10]+[22*1],x
	dex
	bpl	sht_lp1
	rts

string:
	dc.b	"       PRESENTS       "
	dc.b	"                      "
	dc.b	"   VIC-TRACKER 1.0!   "

;**************************************************************************
;**
;** Display scroll
;**
;******

Showscroll:
	ldx	#0
ssc_lp1:
	txa
	clc
	adc	#$c0-30
	sta	$1000+[30*10]+[22*4]+[30*2],x
	lda	#SCROLL_TEXTCOL
	sta	$9400+[30*10]+[22*4]+[30*2],x
	inx
	cpx	#30
	bne	ssc_lp1

	ldx	#0
	txa
ssc_lp2:
	sta	$1e00-[30*8],x
	inx
	cpx	#30*8
	bne	ssc_lp2
	
	rts

scrollflag:
	dc.b	0
	
doscroll:
	lda	scrollflag
	beq	dsc_ex1
	ldx	#8-1
dsc_lp1:	
	asl	nextchar,x
	rol	$1df8,x
	rol	$1df0,x
	rol	$1de8,x
	rol	$1de0,x
	rol	$1dd8,x
	rol	$1dd0,x
	rol	$1dc8,x
	rol	$1dc0,x
	rol	$1db8,x
	rol	$1db0,x
	rol	$1da8,x
	rol	$1da0,x
	rol	$1d98,x
	rol	$1d90,x
	rol	$1d88,x
	rol	$1d80,x
	rol	$1d78,x
	rol	$1d70,x
	rol	$1d68,x
	rol	$1d60,x
	rol	$1d58,x
	rol	$1d50,x
	rol	$1d48,x
	rol	$1d40,x
	rol	$1d38,x
	rol	$1d30,x
	rol	$1d28,x
	rol	$1d20,x
	rol	$1d18,x
	rol	$1d10,x
	dex
	bpl	dsc_lp1		

	dec	scrcount
	lda	scrcount
	beq	dsc_skp1
dsc_ex1:	
	rts

dsc_skp1:
	lda	#8
	sta	scrcount

textptr:
	lda	scrolltext
	bne	dsc_skp2
	lda	#<scrolltext
	sta	textptr+1
	lda	#>scrolltext
	sta	textptr+2
	jmp	textptr

dsc_skp2:
	inc	textptr+1
	bne	dsc_skp3
	inc	textptr+2
dsc_skp3:
	and	#$3f	
	sta	$fb
	lda	#0
	asl	$fb
	rol
	asl	$fb
	rol
	asl	$fb
	rol
	clc
	adc	#$1e
	sta	$fc

	ldy	#8-1
dsc_lp2:
	lda	($fb),y
	sta	nextchar,y
	dey
	bpl	dsc_lp2

	rts
	

scrcount:
	dc.b	8
nextchar:	
	ds.b	8,0
	
;**************************************************************************
;**
;** Frg data
;**
;******
	org	$2700
Buffer1:
	ds.b	buflen,0
Buffer2:
	ds.b	buflen,LOGO_BGCOL
Tab1:
	dc.b	14
	dc.b	10
	dc.b	7
	dc.b	15
	dc.b	7
	dc.b	10
	dc.b	$ff
Tab2:
	dc.b	2,8,10,7,15,1,15,7,10,8,2,0,$ff


scrolltext:
	dc.b	"ALPHA RANGE 89 PRESENTS VIC-TRACKER 1.0!  THIS LONG "
	dc.b	"AWAITED MUSICIANS TOOL WAS ORIGINALLY WRITTEN IN 1994 "
	dc.b	"BUT NOT RELEASED TO THE PUBLIC.  THIS 2001-07-21 RELEASE "
	dc.b	"ADDS A NUMBER OF IMPROVEMENTS AND IMPORTANT FEATURES OVER "
	dc.b	"THE ORIGINAL VERSION.    VIC-TRACKER 1.0 IS THE DEFINITE "
	dc.b	"TOOL OF CHOICE FOR THE PROFESSIONAL VIC-20 COMPOSER!    "
	dc.b	"WE WOULD LIKE TO THANK OUR BETA TESTERS WHO ARE TOO "
	dc.b	"NUMEROUS TO MENTION HERE.   THEY HAVE PROVIDED US WITH "
	dc.b	"IMPORTANT FEEDBACK WHICH WAS MOST HELPFUL IN COMPLETING "
	dc.b	"THE FINAL PRODUCT.                 "
	dc.b	0
scrolltextend:
	echo	"scrolltext",scrolltext,"-",scrolltextend
	
	IF	MUSIC
	include proj:cbm/20Play/Player.asm
	ENDIF
