/*************************************************************************
**
** FILE  vtcomp.c
** Copyright (c) 1994, 2001 Daniel Kahlin
** Written by Daniel Kahlin <daniel@kahlin.net>
**
** DESCRIPTION
**   usage: vtcomp <infile> <outfile>
**
******/

#include <stdio.h>
#include <stdlib.h>
#include <error.h>
#include <string.h>
#include <unistd.h>

/* this is a quick fix for the amiga types */
#define UBYTE u_int8_t
#define UWORD u_int16_t
#define ULONG u_int32_t
#define LONG int32_t
#define TRUE -1L
#define FALSE 0L

#include "tune.h"

void ConvertTune(UBYTE *filename,struct Tune *tune);
void fdumpbytes(FILE *fp,UBYTE *data,UWORD num,UWORD numperrow);
void fdumplabels(FILE *fp,UBYTE *label,UWORD num,UWORD numperrow);
UBYTE *LoadFile(UBYTE *filename);
void PackPattern(FILE *fp, struct Pattern *pattern);
void PackPattlist(FILE *fp, struct Tab *tab, UBYTE startstep, UBYTE endstep);
UWORD UnpackPattern(FILE *fp, UBYTE *buffer, UBYTE *destbuffer);
UWORD UnpackPattlist(FILE *fp, UBYTE *buffer, UBYTE *destbuffer);

static UBYTE pitchconvtab[]={
    0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
    0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0xff,0xff,0xff,0xff,
    0x0d,0x0e,0x0f,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0xff,0xff,0xff,0xff,
    0x19,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f,0x20,0x21,0x22,0x23,0x24,0xff,0xff,0xff,0xff,
    0x25,0x26,0x27,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
    0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
    0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
    0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

static UBYTE *effect[]={
    "None",
    "Portamento Up",
    "Portamento Down",
    "Arpeggio",
    "N/A",
    "Portamento Up (Slow)",
    "Portamento Down (Slow)",
    "N/A",
    "N/A",
    "N/A",
    "N/A",
    "N/A",
    "N/A",
    "N/A",
    "N/A",
    "N/A"
};

#define DBGF_NONE 0
#define DBGF_FULL 1<<0
#define DBGF_MEDIUM 1<<1

ULONG debug=DBGF_NONE;

/*************************************************************************
**
** Main routine
** 
**
******/
int main(int argc, char *argv[])
{
    struct Tune *tune=NULL;
    UBYTE *loadname=NULL;
    UBYTE *savename=NULL;

    puts(PACKAGE " " VERSION " - Copyright (c) 1994, 2001 Daniel Kahlin <daniel@kahlin.net>");
    if (argc<3) {
	printf("usage: vtcomp <infile> <outfile>\n");
	exit(1);
    }
		
    loadname=argv[1];	
    savename=argv[2];

    tune=(struct Tune *)LoadFile(loadname);
    if (!tune) {
	printf("couldn't load infile!\n");
	exit(1);
    }
    if (tune->ID1==TUNE_ID1 && tune->ID2==TUNE_ID2) {
	printf("file: '%s' \n",loadname);
	ConvertTune(savename,tune);
	printf("wrote to file '%s'.\n",savename);
    } else {
	puts("not a vic-tracker tune\n");
	exit(1);
    }

/* all ok exit */
    free(tune);
    exit(0);
}



/*************************************************************************
**
** ConvertTune
**
******/
void ConvertTune(UBYTE *filename,struct Tune *tune)
{
    FILE *fp;
    int i,j,k,l;
    UBYTE pattnum=0;
    UBYTE pitch=0;
    UBYTE param=0;
    UBYTE startstep=tune->SongDef[0].StartStep;
    UBYTE endstep=tune->SongDef[0].EndStep;
    UBYTE pitchtab[256];
    UBYTE paramtab[256];
    UBYTE paramsubst[256];
    UBYTE patterntab[256];
    UBYTE patternsubst[256];
    UBYTE effecttab[16];
    UBYTE arptab[16];
    UBYTE arpsubst[16];

    printf(" - vic-tracker %d.%d tune\n",tune->Version,tune->Revision);
    printf(" - number of songs: %d\n",tune->SongNum);

    for (i=0; i<tune->SongNum; i++) {
	printf("song %d:\n",i);
	printf(" - StartStep %02x\n",tune->SongDef[i].StartStep);
	printf(" - EndStep %02x\n",tune->SongDef[i].EndStep);
	printf(" - StartSpeed %02x\n",tune->SongDef[i].StartSpeed);
    }

    if (!(fp=fopen(filename,"w"))) {
	puts("couldn't open file");
	return;
    }


/* Generate header */
    fprintf(fp,
	    "\n"
	    ";**************************************************************************\n"
	    ";**\n"
	    ";** vic-tracker module\n"
	    ";**\n"
	    ";** This file was generated by:\n"
	    ";**   " PACKAGE " " VERSION " by Daniel Kahlin <daniel@kahlin.net>\n"
	    ";**\n"
	    ";******\n");

/* clear tables */
    for (i=0; i < 256; i++) {
	pitchtab[i]=0;
	paramtab[i]=0;
	paramsubst[i]=0;
	patterntab[i]=0;
	patternsubst[i]=0;
    }
    for (i=0; i < 16; i++) {
	effecttab[i]=0;
	arpsubst[i]=0;
	arptab[i]=0;
    }

/* make map of pitches (stmma 1-3) */
    for (j=0; j < 3; j++) {
	for (i=startstep; i <= endstep; i++) {
	    for (k=0; k < TUNE_PATTLEN; k++) {
		pitchtab[tune->Pattern[tune->Tab[j].Data[i]].Note[k].Pitch]=1;
	    }
	}	
    }


/* make map of params (stmma 1-5) */
    for (j=0; j < 5; j++) {
	for (i=startstep; i <= endstep; i++) {
	    for (k=0; k < TUNE_PATTLEN; k++) {
		paramtab[tune->Pattern[tune->Tab[j].Data[i]].Note[k].Param]=1;
		effecttab[(tune->Pattern[tune->Tab[j].Data[i]].Note[k].Param)>>4]=1;
	    }
	}	
    }


/* Gr patternmap! (stmma 1-5) */
    for (j=0; j < 5; j++) {
	for (i=startstep; i <= endstep; i++) {
	    if (j<3)
		patterntab[tune->Tab[j].Data[i]]=1;
	    else
		patterntab[tune->Tab[j].Data[i]]=2;
	}	
    }


/* Gr patternsubst! */
    j=0;
    for (i=0; i < 256; i++) {
	if (patterntab[i])
	    patternsubst[i]=j++;
    }
    pattnum=j;	


/* Kolla vilka effekter som r anvnda */
    fprintf(fp,";used effects:\n");
    for (i=0; i < 16; i++) {
	if (effecttab[i])
	    fprintf(fp,"; - %01x %s\n",i,effect[i]);
    }


/* Kolla vilka Arpeggios som r anvnda */
    j=0;
    fprintf(fp,";used arpeggios:\n;  - ");
    for (i=0; i < 16; i++) {
	if (paramtab[i+0x30]) {
	    arptab[i]=1;
	    arpsubst[i]=j;
	    if (j!=0)
		fprintf(fp,", ");
	    fprintf(fp,"%01x",i);
	    j++;
	}
    }
    fprintf(fp,"\n");


/* Generera Arpeggios */
    fprintf(fp,"pl_Arpeggios:\n");
    for (i=0; i < 16; i++) {
	if (arptab[i]) {
	    fdumpbytes(fp,(UBYTE *)&tune->Arpeggio[i],8,8);
	}
    }

/* substituera noter (stmma 1-3) */
    for (i=0; i < 256; i++) {
	if (patterntab[i]==1) {
	    for (k=0; k < TUNE_PATTLEN; k++) {
		pitch=tune->Pattern[i].Note[k].Pitch;
		tune->Pattern[i].Note[k].Pitch=(pitchconvtab[pitch&0x7f])|(pitch&0x80);
	    }			
	}
    }

/* substituera effekter (stmma 1-5) */
    for (i=0; i < 256; i++) {
	if (patterntab[i]) {
	    for (k=0; k < TUNE_PATTLEN; k++) {
		param=tune->Pattern[i].Note[k].Param;
		switch (param >> 4) {
		case 3:
		    param=(param&0xf0)+arpsubst[param&0x0f];
		    break;
		default:
		    break;
		}
		tune->Pattern[i].Note[k].Param=param;
	    }			
	}
    }


/* substituera i pattlist */
    for (j=0; j < 5; j++) {
	for (l=startstep; l<=endstep; l++)
	    tune->Tab[j].Data[l]=patternsubst[tune->Tab[j].Data[l]];
    }


    fprintf(fp,"pl_StartSpeed:\n\tdc.b\t%d\n",tune->SongDef[0].StartSpeed);


/* Generera pattlist */
    for (j=0; j < 5; j++) {
	fprintf(fp,"pl_Tab%d:\n",j);
	printf("packing pattlist %d...",j);
	PackPattlist(fp,&tune->Tab[j],startstep,endstep);
    }


    fprintf(fp,"pl_Patterns:\n");
    fdumplabels(fp,"pl_Pattern",pattnum,6);

/* Generera patterns */
    for (i=0; i < 256; i++) {
	if (patterntab[i]) {

	    fprintf(fp,"pl_Pattern%d:\n",patternsubst[i]);
	    printf("packing pattern %02x...",patternsubst[i]);
	    PackPattern(fp,&(tune->Pattern[i]));
	}
    }


/* Terminate */
    fprintf(fp,"; eof\n");
    fclose(fp);
}


/*************************************************************************
**
** PackPattlist and output to file
**
******/
void PackPattlist(FILE *fp, struct Tab *tab, UBYTE startstep, UBYTE endstep)
{
    UBYTE pattlistbuffer[256];
    UBYTE pattlistbuffer2[256];
    UBYTE datacache[256];
    UBYTE state=16;
    UBYTE count=0;
    UBYTE index=0;
    int i,j;
    UBYTE run=TRUE;
    UWORD len=0;


    i=startstep;
    while (run) {
	if (i<=(endstep)) {
	    datacache[count]=tab->Data[i];
	    switch(state) {
	    case 16:
		state=0;
		break;
	    case 0:
		if (datacache[count]==datacache[count-1])
		    {state=8; break;}
		if (datacache[count]!=datacache[count-1])
		    {state=9; break;}
		state|=32;
		break;
	    case 8:
		if (datacache[count]==datacache[count-1])
		    {state=8; break;}
		state|=32;
		break;
	    case 9:
		if (datacache[count]!=datacache[count-1])
		    {state=9; break;}
		count--;
		i--;
		state|=32;
		break;
	    default:
		puts("error");
		break;
	    }				
	} else {
	    state|=32;
	    run=FALSE;
	}
	
	if (state&32) {
	    switch (state&0x03) {
	    case 0:
		pattlistbuffer[index++]=0x80+(count-1);
		pattlistbuffer[index++]=datacache[0];
		break;
	    case 1:
		for (j=0; j<count; j++)
		    pattlistbuffer[index++]=datacache[j];
		break;
	    default:
		puts("unknown state");
		break;
	    }
	    count=0;
	    state=16;
	} else {
	    count++;
	    i++;
	}
    }
    pattlistbuffer[index++]=0xc0;

    fdumpbytes(fp,pattlistbuffer,index,16);

    len=UnpackPattlist(fp,pattlistbuffer,pattlistbuffer2);

    if ((!memcmp(&(tab->Data[startstep]),pattlistbuffer2,(endstep-startstep)+1)) && len==(endstep-startstep)+1)
	printf("unpacks ok. ");
    else
	printf("couldn't unpack! ");

    printf("(size: $%02x)\n",index);

}

/*************************************************************************
**
** Unpackpattlist and output to file
**
******/
UWORD UnpackPattlist(FILE *fp, UBYTE *buffer, UBYTE *destbuffer)
{
    UBYTE data;
    int i=0,j=0;
    UBYTE index=0;
    UBYTE run=TRUE;

/* clear tables */
    for (i=0; i < 256; i++) {
	destbuffer[i]=0;
    }

    i=0;
    while(run) {
	data=buffer[i++];

	switch (data & 0xc0) {
	case 0x00:
	case 0x40:
	    destbuffer[index++]=data&0x7f;
	    break;
	case 0x80:
	    for (j=0; j<=(data&0x3f); j++)
		destbuffer[index++]=buffer[i];
	    i++;
	    break;
	case 0xc0:
	    run=FALSE;
	    break;
	default:
	    break;
	}

    }
	

    return(index);
}


/*************************************************************************
**
** Packpattern and output to file
**
******/
void PackPattern(FILE *fp, struct Pattern *pattern)
{
    UBYTE patternbuffer[256];
    UBYTE patternbuffer2[256];
    UBYTE pitchcache[256],paramcache[256];
    UBYTE state=16;
    UBYTE count=0;
    UBYTE index=0;
    int	i,j;
    UBYTE run=TRUE;
    UWORD len=0;


    i=0;
    while (run) {
	if (i<TUNE_PATTLEN) {
	    pitchcache[count]=pattern->Note[i].Pitch;
	    paramcache[count]=pattern->Note[i].Param;
	    switch (state) {
	    case 16:
		if (pitchcache[count]==0x00 && paramcache[count]==0x00)
		    { state=0; break;}
		if (paramcache[count]==0x00)
		    { state=1; break;}	
		if (pitchcache[count]==0x80)
		    { state=2; break;}
		state=3;
		break;
	    case 0:
		if (pitchcache[count]==0x00 && paramcache[count]==0x00)
		    { state=0+8; break;}
		state|=32;
		break;
	    case 1:
		if (pitchcache[count]==pitchcache[count-1] && paramcache[count]==0x00)
		    { state=1+8; break;}
		if (pitchcache[count]!=pitchcache[count-1] && paramcache[count]==0x00)
		    { state=5+8; break;}
		state|=32;
		break;
	    case 2:
		if (pitchcache[count]==0x80 && paramcache[count]==paramcache[count-1])
		    { state=2+8; break;}
		if (pitchcache[count]==0x80 && paramcache[count]!=paramcache[count-1])
		    { state=6+8; break;}
		state|=32;
		break;
	    case 3:
		if (pitchcache[count]==pitchcache[count-1] && paramcache[count]==paramcache[count-1])
		    { state=3+8; break;}
		if (pitchcache[count]!=pitchcache[count-1] || paramcache[count]!=paramcache[count-1])
		    { state=7+8; break;}
		state|=32;
		break;
	    case 8:
		if (pitchcache[count]==0x00 && paramcache[count]==0x00)
		    { state=0+8; break;}
		state|=32;
		break;
	    case 9:
		if (pitchcache[count]==pitchcache[count-1] && paramcache[count]==0x00)
		    { state=1+8; break;}
		state|=32;
		break;
	    case 10:
		if (pitchcache[count]==0x80 && paramcache[count]==paramcache[count-1])
		    { state=2+8; break;}
		state|=32;
		break;
	    case 11:
		if (pitchcache[count]==pitchcache[count-1] && paramcache[count]==paramcache[count-1])
		    { state=3+8; break;}
		state|=32;
		break;
	    case 13:
		if (pitchcache[count]!=pitchcache[count-1] && paramcache[count]==0x00)
		    { state=5+8; break;}
		state|=32;
		break;
	    case 14:
		if (pitchcache[count]==0x80 && paramcache[count]!=paramcache[count-1])
		    { state=6+8; break;}
		state|=32;
		break;
	    case 15:
		if (pitchcache[count]!=pitchcache[count-1] || paramcache[count]!=paramcache[count-1])
		    { state=7+8; break;}
		state|=32;
		break;
	    default:
		puts("packer: unknown state");			
		break;
	    }
	    if (debug&DBGF_FULL)
		printf("%02x: %02x %02x state %d  count %d\n",i,pitchcache[count],paramcache[count],state,count);
	} else {
	    state|=32;
	    run=FALSE;
	}

	if (state&32) {
	    patternbuffer[index++]=((state&0x07)<<5)|(count-1);
	    switch (state&0x07) {
	    case 0:
		break;
	    case 1:
		patternbuffer[index++]=pitchcache[0];
		break;
	    case 2:
		patternbuffer[index++]=paramcache[0];
		break;
	    case 3:
		patternbuffer[index++]=pitchcache[0];
		patternbuffer[index++]=paramcache[0];
		break;
	    case 4:
		puts("illegal state");
		break;
	    case 5:
		for (j=0; j<count; j++)
		    patternbuffer[index++]=pitchcache[j];
		break;
	    case 6:
		for (j=0; j<count; j++)
		    patternbuffer[index++]=paramcache[j];
		break;
	    case 7:
		for (j=0; j<count; j++) {
		    patternbuffer[index++]=pitchcache[j];
		    patternbuffer[index++]=paramcache[j];
		}
		break;
	    default:
		puts("unknown state");
		break;
	    }
	    count=0;
	    state=16;
	} else {
	    count++;
	    i++;
	}
    }

    patternbuffer[index++]=0x80;

/*
** om orimlig lngd
** fixa en klon.
*/
    if (index>(TUNE_PATTLEN*2+2)) {
	index=0;
	patternbuffer[index++]=0xe0+(TUNE_PATTLEN-1);
	for (j=0; j<TUNE_PATTLEN; j++) {
	    patternbuffer[index++]=pattern->Note[i].Pitch;
	    patternbuffer[index++]=pattern->Note[i].Param;
	}
	patternbuffer[index++]=0x80;
    }


    fdumpbytes(fp,patternbuffer,index,16);

    len=UnpackPattern(fp,patternbuffer,patternbuffer2);

    if ((!memcmp(pattern,patternbuffer2,TUNE_PATTLEN*2)) && len==TUNE_PATTLEN*2)
	printf("unpacks ok. ");
    else
	printf("couldn't unpack! ");

    printf("(size: $%02x)\n",index);
}


/*************************************************************************
**
** Unpackpattern and output to file
**
******/
UWORD UnpackPattern(FILE *fp, UBYTE *buffer, UBYTE *destbuffer)
{
    UBYTE data;
    UBYTE t=0;
    int i=0,j=0;
    UBYTE index=0;
    UBYTE run=TRUE;

/* clear tables */
    for (i=0; i < 256; i++) {
	destbuffer[i]=0;
    }

    i=0;

    while(run) {
	data=buffer[i++];

	t=data&0x1f;
	switch ((data>>5)&0x07) {
	case 0:
	    for (j=0; j<=t; j++) {
		destbuffer[index++]=0;
		destbuffer[index++]=0;
	    }
	    break;
	case 1:
	    for (j=0; j<=t; j++) {
		destbuffer[index++]=buffer[i];
		destbuffer[index++]=0;
	    }
	    i++;
	    break;
	case 2:
	    for (j=0; j<=t; j++) {
		destbuffer[index++]=0x80;
		destbuffer[index++]=buffer[i];
	    }
	    i++;
	    break;
	case 3:
	    for (j=0; j<=t; j++) {
		destbuffer[index++]=buffer[i];
		destbuffer[index++]=buffer[i+1];
	    }
	    i+=2;
	    break;
	case 4:
	    run=FALSE;
	    break;
	case 5:
	    for (j=0; j<=t; j++) {
		destbuffer[index++]=buffer[i++];
		destbuffer[index++]=0;
	    }
	    break;
	case 6:
	    for (j=0; j<=t; j++) {
		destbuffer[index++]=0x80;
		destbuffer[index++]=buffer[i++];
	    }
	    break;
	case 7:
	    for (j=0; j<=t; j++) {
		destbuffer[index++]=buffer[i++];
		destbuffer[index++]=buffer[i++];
	    }
	    break;
	default:
	    puts("Unpack: error");
	    break;
	}
    }
    
    return(index);
}





/*************************************************************************
**
** Make assembly dc.b statements!
**
******/
void fdumpbytes(FILE *fp,UBYTE *data,UWORD num,UWORD numperrow)
{
    int	i;

    for	(i=0; i < num; i++) {
	if ((i % numperrow)==0)
	    fprintf(fp,"\tdc.b\t");
	
	if ((i % numperrow)!=numperrow-1 && i!=(num-1))
	    fprintf(fp,"$%02x,",data[i]);
	else
	    fprintf(fp,"$%02x\n",data[i]);
    }
}


/*************************************************************************
**
** Make assembly dc.w statements!
**
******/
void fdumplabels(FILE *fp,UBYTE *label,UWORD num,UWORD numperrow)
{
    int	i;

    for (i=0; i < num; i++) {
	if ((i % numperrow)==0)
	    fprintf(fp,"\tdc.w\t");
	
	if ((i % numperrow)!=numperrow-1 && i!=(num-1))
	    fprintf(fp,"%s%d,",label,i);
	else
	    fprintf(fp,"%s%d\n",label,i);
    }
}



/*************************************************************************
**
** Ladda fil till buffer
**
******/
UBYTE *LoadFile(UBYTE *filename)
{
    UBYTE *ptr;
    int size;
    FILE *fin;

    if (!(fin=fopen(filename,"rb")))
	return(NULL);
    fseek(fin,0,SEEK_END);
    if (ferror(fin))
	return(NULL);

    size=ftell(fin);
    ptr=malloc(size);

    fseek(fin,0,SEEK_SET);
    fread(ptr,size,1,fin);

    if (ferror(fin))
	return(NULL); 
    fclose(fin);

    return(ptr);
}
/* eof */
